import sys

import typer
import importlib


app = typer.Typer()

def import_class(mod_str: str):
    split_import_string = mod_str.split(".")
    import_path = ".".join(split_import_string[:-1])
    import_class_string = split_import_string[-1]
    mod = importlib.import_module(import_path)
    metrics_class = getattr(mod, import_class_string)
    return metrics_class


@app.command()
def luigi(task_path: str):
    try:
        import luigi
        from sirena.luigi.sample_dag import build_task_tree
    except ImportError:
        typer.echo("Luigi not installed. Please `pip install luigi` to use `sirena luigi`")
        sys.exit(1)

    luigi_task = import_class(task_path)
    fc = build_task_tree(luigi_task())

    typer.echo(fc.render())

@app.command()
def pydantic():
    typer.echo("Not implemented")

if __name__ == "__main__":
    app()


import enum
from typing import Dict, Optional, List, Set, Tuple, Type, Union

import pydantic

from sirena.base import BaseDiagram, BaseNode, Edge


class BaseEntity(BaseNode):
    attributes: Optional[Dict[str, str]] = pydantic.Field(default={})

    def __hash__(self):
        return hash(self.id)

    def render(self) -> str:
        return self.id


class EntityFactory:
    def __init__(self):
        self._registry: Dict[str, BaseEntity] = {}

    def __call__(self, id, attributes=None, *args, **kwargs):
        if id in self._registry:
            return self._registry[id]
        else:
            return BaseEntity(id=id, attributes=attributes or {}, *args, **kwargs)


class Cardinality(enum.Enum):
    ZERO_OR_ONE: Tuple[str, str] = ("|o ", "o|")
    EXACTLY_ONE: Tuple[str, str] = ("||", "||")
    ZERO_OR_MORE: Tuple[str, str] = ("}o", "o{")
    ONE_OR_MORE: Tuple[str, str] = ("}|", "|{")


class BaseRelationship(pydantic.BaseModel):
    left: BaseEntity
    right: BaseEntity
    left_cardinality: Cardinality
    right_cardinality: Cardinality

    def __hash__(self):
        return (
            hash(self.left)
            + hash(self.right)
            + hash(self.left_cardinality)
            + hash(self.right_cardinality)
        )

    def _render_line(self):
        return f"{self.left_cardinality.value[0]}--{self.right_cardinality.value[1]}"

    def render(self):
        return f"{self.left.render()} {self._render_line()} {self.right.render()}"


class ERDRenderer:
    def __init__(self, erd: "ERD"):
        self.erd: "ERD" = erd

    def render(self):
        section_string = "erDiagram"

        entities = []
        for entity in self.erd.entities:
            entities.append(entity.render())

        relationships = []
        for relationship in self.erd.relationships:
            relationships.append(relationship.render())

        joined_renders = "\n\t".join([x for x in entities])
        joined_renders = (
            joined_renders + "\n\t" + "\n\t".join([x for x in relationships])
        )

        return f"{section_string}\n\t{joined_renders}"


class ERD(BaseDiagram):
    renderer: Type[ERDRenderer] = ERDRenderer
    items: Set[Union[BaseRelationship, BaseEntity]] = {}

    @property
    def entities(self) -> Set[BaseEntity]:
        return {i for i in self.items if isinstance(i, BaseEntity)}

    @property
    def relationships(self) -> Set[BaseRelationship]:
        return {i for i in self.items if isinstance(i, BaseRelationship)}

    def __add__(self, other):
        if isinstance(other, BaseRelationship) or isinstance(other, BaseEntity):
            self.items.add(other)
        else:
            raise ValueError(f"Can only accept BaseRelationships & BaseEntities")
        return self

    def render(self):
        return self.renderer(self).render()


Entity = EntityFactory()

ERD.update_forward_refs(**locals())


erd = ERD()

f = []
for field in ERD.__fields__.values():
    f.append(field)



from typing import List, Union, Type, Optional

import pydantic


class BaseEdgeRenderer:
    def __init__(self, edge_data: "Edge"):
        self._edge = edge_data

    def render(self):
        raise NotImplemented


class BaseDiagramRenderer:
    def __init__(self, diagram: "BaseDiagram"):
        self._diagram = diagram

    def render(self):
        raise NotImplementedError


class BaseNode(pydantic.BaseModel):
    id: str
    text: Optional[str]

    def render(self) -> str:
        return self.shape.value.format(self.id, self.text or self.id)


class Edge(pydantic.BaseModel):
    start: BaseNode
    end: BaseNode
    edge_renderer: Type[BaseEdgeRenderer]

    def render(self) -> str:
        return self.edge_renderer(self).render()


class BaseDiagram(pydantic.BaseModel):
    items: List[Union[BaseNode, Edge]] = []
    renderer: Type[BaseDiagramRenderer] = BaseDiagramRenderer

    def __add__(self, other: Union["BaseNode", "Edge"]):
        self.items.append(other)
        return self

    def render(self):
        return self.renderer(self).render()


BaseNode.update_forward_refs()
Edge.update_forward_refs()



from typing import Optional, Type, Tuple

import enum

from sirena.base import BaseEdgeRenderer, BaseDiagramRenderer, BaseDiagram, Edge, BaseNode


class NodeShapes(enum.Enum):
    ROUNDED_RECT: str = "{}({})"
    STADIUM: str = "{}([{}])"
    SUBROUTINE: str = "{}[[{}]]"
    CYLINDER: str = "{}[({})]"
    CIRCLE: str = "{}(({}))"
    ASYMMETRIC: str = "{}>{}]"
    RHOMBUS: str = "{}{{{}}}"
    HEXAGON: str = "{}{{{{{}}}}}"
    PARALLELOGRAM: str = "{}[/{}/]"
    PARALLELOGRAM_ALT: str = "{}[\{}\]"
    TRAPEZOID: str = "{}[/{}\]"
    TRAPEZOID_ALT: str = "{}[\{}/]"


class ArrowTypes(enum.Enum):
    X: Tuple[str, str] = ("x", "x")
    O: Tuple[str, str] = ("0", "0")
    ARROW: Tuple[str, str] = ("<", ">")


class Symbol(BaseNode):
    shape: NodeShapes = NodeShapes.ROUNDED_RECT

    def render(self) -> str:
        return self.shape.value.format(self.id, self.text or self.id)


class LineStyle(enum.Enum):
    NORMAL: str = "normal"
    THICK: str = "thick"
    DOTTED: str = "dotted"


class LineDirection(enum.Enum):
    NONE: str = "none"
    UNIDIRECTIONAL: str = "unidirectional"
    MULTIDIRECTIONAL: str = "multidirectional"




class FlowChartEdgeRenderer(BaseEdgeRenderer):
    _edge: 'FlowChartEdge'

    def _render_start_node(self):
        return self._edge.start.render()

    def _render_end_node(self):
        return self._edge.end.render()

    def _render_text(self) -> str:
        if self._edge.text:
            return f"|{self._edge.text}|"

        return ""

    def _render_start_arrow(self) -> str:
        if self._edge.direction == "multi":
            return f"{self._edge.arrow_type.value[0]}"

        return ""

    def _render_end_arrow(self) -> str:
        if self._edge.direction == "multi" or self._edge.direction == "uni":
            return f"{self._edge.arrow_type.value[1]}"

        return ""

    def _render_line(self):

        if self._edge.line_style == LineStyle.NORMAL:
            return f"{self._render_start_arrow()}-{(self._edge.length - 1) * '-'}->{self._render_text()}"

        if self._edge.line_style == LineStyle.DOTTED:
            return f"{self._render_start_arrow()}-{(self._edge.length) * '.'}->{self._render_text()}"

        if self._edge.line_style == LineStyle.THICK:
            return f"{self._render_start_arrow()}={(self._edge.length - 1) * '='}=>{self._render_text()}"

    def render(self):
        return self._render_start_node() + self._render_line() + self._render_end_node()


class FlowChartEdge(Edge):
    text: str = None
    line_style: LineStyle = LineStyle.NORMAL
    arrow_type: ArrowTypes = ArrowTypes.ARROW
    direction: LineDirection = LineDirection.UNIDIRECTIONAL
    length: int = 1
    edge_renderer: Type[BaseEdgeRenderer] = FlowChartEdgeRenderer

class FlowChartRenderer(BaseDiagramRenderer):
    _diagram: 'FlowChart'

    def render(self):

        items_string = "\n\t".join([item.render() for item in self._diagram.items])

        return f"flowchart {self._diagram.orientation}\n\t{items_string}"


class SubGraphRenderer:
    def __init__(self, subgraph: "SubGraph"):
        self._subgraph = subgraph

    def render(self):
        items_string = "\n\t".join([item.render() for item in self._subgraph.items])

        return f"subgraph {self._subgraph.id}\n\t{items_string}\n\tend"


class SubGraph(BaseDiagram):
    renderer: Type[SubGraphRenderer] = SubGraphRenderer
    title: Optional[str] = None
    id: str


class FlowChart(BaseDiagram):
    renderer: Type[FlowChartRenderer] = FlowChartRenderer
    orientation: str = "LR"


# a = Node(id="a", text="test")
# b = Node(id="b")
# c = Node(id="c")
#
# sg = SubGraph(id="test")
# sg + Node(id="e")
#
#
# e = FlowChartEdge(start=a, end=b, direction=LineDirection.UNIDIRECTIONAL)
# e2 = FlowChartEdge(start=a, end=c, text="blah", length=3, line_style=LineStyle.DOTTED)
#
#
# fc = (
#     FlowChart(orientation="TD")
#     + e
#     + e2
#     + FlowChartEdge(start=b, end=c, line_style=LineStyle.THICK)
#     + sg
#     + FlowChartEdge(start=b, end=Node(id="e"))
# )
#
# print(fc.render())





import random as rnd
import time

import luigi

from sirena.flowchart.shapes import FlowChart,Symbol,FlowChartEdge

class Configuration(luigi.Task):
    seed = luigi.IntParameter()

    def output(self):
        """
        Returns the target output for this task.
        In this case, a successful execution of this task will create a file on the local filesystem.
        :return: the target output for this task.
        :rtype: object (:py:class:`luigi.target.Target`)
        """
        return luigi.LocalTarget('/tmp/Config_%d.txt' % self.seed)

    def run(self):
        time.sleep(5)
        rnd.seed(self.seed)

        result = ','.join(
            [str(x) for x in rnd.sample(list(range(300)), rnd.randint(7, 25))])
        with self.output().open('w') as f:
            f.write(result)


class Data(luigi.Task):
    magic_number = luigi.IntParameter()

    def output(self):
        """
        Returns the target output for this task.
        In this case, a successful execution of this task will create a file on the local filesystem.
        :return: the target output for this task.
        :rtype: object (:py:class:`luigi.target.Target`)
        """
        return luigi.LocalTarget('/tmp/Data_%d.txt' % self.magic_number)

    def run(self):
        time.sleep(1)
        with self.output().open('w') as f:
            f.write('%s' % self.magic_number)


class Dynamic(luigi.Task):
    seed = luigi.IntParameter(default=1)

    def output(self):
        """
        Returns the target output for this task.
        In this case, a successful execution of this task will create a file on the local filesystem.
        :return: the target output for this task.
        :rtype: object (:py:class:`luigi.target.Target`)
        """
        return luigi.LocalTarget('/tmp/Dynamic_%d.txt' % self.seed)

    def run(self):
        # This could be done using regular requires method
        config = self.clone(Configuration)
        yield config

        with config.output().open() as f:
            data = [int(x) for x in f.read().split(',')]

        # ... but not this
        data_dependent_deps = [Data(magic_number=x) for x in data]
        yield data_dependent_deps

        with self.output().open('w') as f:
            f.write('Tada!')

def get_edges_items(task: luigi.Task):

    items = [Symbol(id=task.get_task_family())]

    for t in task.deps():
        items.append(FlowChartEdge(end=Symbol(id=task.get_task_family()),start=Symbol(id=t.get_task_family())))
        _i= get_edges_items(t)
        items.extend(_i)
    return items


def build_task_tree(task):
    items = get_edges_items(task)

    return FlowChart(items=items)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sirena', 'sirena.bin', 'sirena.erd', 'sirena.flowchart', 'sirena.luigi']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.9.0,<2.0.0', 'tox>=3.24.5,<4.0.0', 'typer>=0.4.1,<0.5.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata==0.12'],
 'luigi': ['luigi>=3.0.3,<4.0.0']}

entry_points = \
{'console_scripts': ['sirena = sirena.bin.cli:app']}

setup_kwargs = {
    'name': 'sirena',
    'version': '0.1.1.post2',
    'description': '',
    'long_description': None,
    'author': 'Sam Phinizy',
    'author_email': 'nipper@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Optional
from sirena.flowchart.shapes import FlowChart,Symbol,FlowChartEdge

import luigi
from random import randint

class GrandParent(luigi.Task):
    pass

class ParentTask(luigi.Task):

    def requires(self):
        return [GrandParent()]

class UncleTask(luigi.Task):

    def requires(self):
        return [GrandParent()]

class Aunties(luigi.Task):
    number = luigi.IntParameter()

    def requires(self):
        return [GrandParent()]


class Child(luigi.Task):

    def requires(self):


        aunts = [Aunties(i) for i in range(10)]
        return [ParentTask(),UncleTask(),*aunts]


def get_edges_items(task: luigi.Task):

    items = [Symbol(id=task.task_id)]

    for t in task.deps():
        items.append(FlowChartEdge(end=Symbol(id=task.task_id),start=Symbol(id=t.task_id)))
        _i= get_edges_items(t)
        items.extend(_i)
    return items


def build_task_tree(task):
    items = get_edges_items(task)

    return FlowChart(items=items)
